# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.model_endpoints import ModelEndpoints
from ..types.model_providers import ModelProviders
from ..types.template_language import TemplateLanguage
from .agent_kernel_request_reasoning_effort import AgentKernelRequestReasoningEffortParams
from .agent_kernel_request_stop import AgentKernelRequestStopParams
from .agent_kernel_request_template import AgentKernelRequestTemplateParams
from .agent_kernel_request_tools_item import AgentKernelRequestToolsItemParams
from .response_format import ResponseFormatParams


class AgentKernelRequestParams(typing_extensions.TypedDict):
    """
    Base class used by both PromptKernelRequest and AgentKernelRequest.

    Contains the consistent Prompt-related fields.
    """

    model: str
    """
    The model instance used, e.g. `gpt-4`. See [supported models](https://humanloop.com/docs/reference/supported-models)
    """

    endpoint: typing_extensions.NotRequired[ModelEndpoints]
    """
    The provider model endpoint used.
    """

    template: typing_extensions.NotRequired[AgentKernelRequestTemplateParams]
    """
    The template contains the main structure and instructions for the model, including input variables for dynamic values. 
    
    For chat models, provide the template as a ChatTemplate (a list of messages), e.g. a system message, followed by a user message with an input variable.
    For completion models, provide a prompt template as a string. 
    
    Input variables should be specified with double curly bracket syntax: `{{input_name}}`.
    """

    template_language: typing_extensions.NotRequired[TemplateLanguage]
    """
    The template language to use for rendering the template.
    """

    provider: typing_extensions.NotRequired[ModelProviders]
    """
    The company providing the underlying model service.
    """

    max_tokens: typing_extensions.NotRequired[int]
    """
    The maximum number of tokens to generate. Provide max_tokens=-1 to dynamically calculate the maximum number of tokens to generate given the length of the prompt
    """

    temperature: typing_extensions.NotRequired[float]
    """
    What sampling temperature to use when making a generation. Higher values means the model will be more creative.
    """

    top_p: typing_extensions.NotRequired[float]
    """
    An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass.
    """

    stop: typing_extensions.NotRequired[AgentKernelRequestStopParams]
    """
    The string (or list of strings) after which the model will stop generating. The returned text will not contain the stop sequence.
    """

    presence_penalty: typing_extensions.NotRequired[float]
    """
    Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the generation so far.
    """

    frequency_penalty: typing_extensions.NotRequired[float]
    """
    Number between -2.0 and 2.0. Positive values penalize new tokens based on how frequently they appear in the generation so far.
    """

    other: typing_extensions.NotRequired[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    Other parameter values to be passed to the provider call.
    """

    seed: typing_extensions.NotRequired[int]
    """
    If specified, model will make a best effort to sample deterministically, but it is not guaranteed.
    """

    response_format: typing_extensions.NotRequired[ResponseFormatParams]
    """
    The format of the response. Only `{"type": "json_object"}` is currently supported for chat.
    """

    reasoning_effort: typing_extensions.NotRequired[AgentKernelRequestReasoningEffortParams]
    """
    Guidance on how many reasoning tokens it should generate before creating a response to the prompt. OpenAI reasoning models (o1, o3-mini) expect a OpenAIReasoningEffort enum. Anthropic reasoning models expect an integer, which signifies the maximum token budget.
    """

    tools: typing_extensions.NotRequired[typing.Sequence[AgentKernelRequestToolsItemParams]]
    attributes: typing_extensions.NotRequired[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    Additional fields to describe the Prompt. Helpful to separate Prompt versions from each other with details on how they were created or used.
    """

    max_iterations: typing_extensions.NotRequired[int]
    """
    The maximum number of iterations the Agent can run. This is used to limit the number of times the Agent model is called.
    """
