# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import typing_extensions
from ..types.model_endpoints import ModelEndpoints
from ..types.model_providers import ModelProviders
from ..types.template_language import TemplateLanguage
from ..types.user_response import UserResponse
from ..types.version_status import VersionStatus
from .agent_response_reasoning_effort import AgentResponseReasoningEffortParams
from .agent_response_stop import AgentResponseStopParams
from .agent_response_template import AgentResponseTemplateParams
from .environment_response import EnvironmentResponseParams
from .evaluator_aggregate import EvaluatorAggregateParams
from .input_response import InputResponseParams
from .response_format import ResponseFormatParams

if typing.TYPE_CHECKING:
    from .agent_response_tools_item import AgentResponseToolsItemParams
    from .monitoring_evaluator_response import MonitoringEvaluatorResponseParams


class AgentResponseParams(typing_extensions.TypedDict):
    """
    Base type that all File Responses should inherit from.

    Attributes defined here are common to all File Responses and should be overridden
    in the inheriting classes with documentation and appropriate Field definitions.
    """

    path: str
    """
    Path of the Agent, including the name, which is used as a unique identifier.
    """

    id: str
    """
    Unique identifier for the Agent.
    """

    directory_id: typing_extensions.NotRequired[str]
    """
    ID of the directory that the file is in on Humanloop.
    """

    model: str
    """
    The model instance used, e.g. `gpt-4`. See [supported models](https://humanloop.com/docs/reference/supported-models)
    """

    endpoint: typing_extensions.NotRequired[ModelEndpoints]
    """
    The provider model endpoint used.
    """

    template: typing_extensions.NotRequired[AgentResponseTemplateParams]
    """
    The template contains the main structure and instructions for the model, including input variables for dynamic values. 
    
    For chat models, provide the template as a ChatTemplate (a list of messages), e.g. a system message, followed by a user message with an input variable.
    For completion models, provide a prompt template as a string. 
    
    Input variables should be specified with double curly bracket syntax: `{{input_name}}`.
    """

    template_language: typing_extensions.NotRequired[TemplateLanguage]
    """
    The template language to use for rendering the template.
    """

    provider: typing_extensions.NotRequired[ModelProviders]
    """
    The company providing the underlying model service.
    """

    max_tokens: typing_extensions.NotRequired[int]
    """
    The maximum number of tokens to generate. Provide max_tokens=-1 to dynamically calculate the maximum number of tokens to generate given the length of the prompt
    """

    temperature: typing_extensions.NotRequired[float]
    """
    What sampling temperature to use when making a generation. Higher values means the model will be more creative.
    """

    top_p: typing_extensions.NotRequired[float]
    """
    An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass.
    """

    stop: typing_extensions.NotRequired[AgentResponseStopParams]
    """
    The string (or list of strings) after which the model will stop generating. The returned text will not contain the stop sequence.
    """

    presence_penalty: typing_extensions.NotRequired[float]
    """
    Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the generation so far.
    """

    frequency_penalty: typing_extensions.NotRequired[float]
    """
    Number between -2.0 and 2.0. Positive values penalize new tokens based on how frequently they appear in the generation so far.
    """

    other: typing_extensions.NotRequired[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    Other parameter values to be passed to the provider call.
    """

    seed: typing_extensions.NotRequired[int]
    """
    If specified, model will make a best effort to sample deterministically, but it is not guaranteed.
    """

    response_format: typing_extensions.NotRequired[ResponseFormatParams]
    """
    The format of the response. Only `{"type": "json_object"}` is currently supported for chat.
    """

    reasoning_effort: typing_extensions.NotRequired[AgentResponseReasoningEffortParams]
    """
    Guidance on how many reasoning tokens it should generate before creating a response to the prompt. OpenAI reasoning models (o1, o3-mini) expect a OpenAIReasoningEffort enum. Anthropic reasoning models expect an integer, which signifies the maximum token budget.
    """

    tools: typing.Sequence["AgentResponseToolsItemParams"]
    """
    List of tools that the Agent can call. These can be linked files or inline tools.
    """

    attributes: typing_extensions.NotRequired[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    Additional fields to describe the Prompt. Helpful to separate Prompt versions from each other with details on how they were created or used.
    """

    max_iterations: typing_extensions.NotRequired[int]
    """
    The maximum number of iterations the Agent can run. This is used to limit the number of times the Agent model is called.
    """

    version_name: typing_extensions.NotRequired[str]
    """
    Unique name for the Agent version. Version names must be unique for a given Agent.
    """

    version_description: typing_extensions.NotRequired[str]
    """
    Description of the version, e.g., the changes made in this version.
    """

    description: typing_extensions.NotRequired[str]
    """
    Description of the Agent.
    """

    tags: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    List of tags associated with the file.
    """

    readme: typing_extensions.NotRequired[str]
    """
    Long description of the file.
    """

    name: str
    """
    Name of the Agent.
    """

    schema: typing_extensions.NotRequired[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    The JSON schema for the Prompt.
    """

    version_id: str
    """
    Unique identifier for the specific Agent Version. If no query params provided, the default deployed Agent Version is returned.
    """

    type: typing_extensions.NotRequired[typing.Literal["agent"]]
    environments: typing_extensions.NotRequired[typing.Sequence[EnvironmentResponseParams]]
    """
    The list of environments the Agent Version is deployed to.
    """

    created_at: dt.datetime
    updated_at: dt.datetime
    created_by: typing_extensions.NotRequired[UserResponse]
    """
    The user who created the Agent.
    """

    committed_by: typing_extensions.NotRequired[UserResponse]
    """
    The user who committed the Agent Version.
    """

    committed_at: typing_extensions.NotRequired[dt.datetime]
    """
    The date and time the Agent Version was committed.
    """

    status: VersionStatus
    """
    The status of the Agent Version.
    """

    last_used_at: dt.datetime
    version_logs_count: int
    """
    The number of logs that have been generated for this Agent Version
    """

    total_logs_count: int
    """
    The number of logs that have been generated across all Agent Versions
    """

    inputs: typing.Sequence[InputResponseParams]
    """
    Inputs associated to the Agent. Inputs correspond to any of the variables used within the Agent template.
    """

    evaluators: typing_extensions.NotRequired[typing.Sequence["MonitoringEvaluatorResponseParams"]]
    """
    Evaluators that have been attached to this Agent that are used for monitoring logs.
    """

    evaluator_aggregates: typing_extensions.NotRequired[typing.Sequence[EvaluatorAggregateParams]]
    """
    Aggregation of Evaluator results for the Agent Version.
    """

    raw_file_content: typing_extensions.NotRequired[str]
    """
    The raw content of the Agent. Corresponds to the .agent file.
    """
